function llf=LCML_pairs(par,W,X,y)


% computes pairwise composite marginal likelihood function (CML_2)


% par denotes Kx1 vector of paramters with par(1) being the spatial paramter rho!!
% W denotes NxN matrix of spatial weight => W should be declared as a sparse matrix!!
% X denotes NxK matrix of regressors
% y denotes Nx1 vector of a binary dependent variable

% the output argument llf denote the composite marginal likelihood function


warning('off', 'MATLAB:rankDeficientMatrix')
warning('off', 'MATLAB:nearlySingularMatrix')
warning('off', 'MATLAB:singularMatrix')



% determine number of observations
N=length(W);

% create Q matrix that allows to write the likelihood as an integral in terms of upper bounds (see Beron and Vijverbeg (2004) - Probit in a Spatial Context: A Monte Carlo Analysis)
Q = spdiags(1-2*y,0,N,N);

% compute vector of truncation thresholds (here we consider SAL model) and precision matrix 
Dinv=speye(N)-par(1)*W;
v=(-Q)*(Dinv\(X*par(2:end)));
H=Q'*(Dinv'*Dinv)*Q;


% compute covariance matrix elements
% SAMD permutation
ppp=symamd(H); % permutation vector
H=H(ppp,ppp);
v=v(ppp);

% Choleksy factorization
L = chol(H)';

% find pairs
W=W(ppp,ppp); % reorder weight matrix
[i,j]=find(W);
pair=[i j];
pair(pair(:,1)>pair(:,2),:)=[pair(pair(:,1)>pair(:,2),2) pair(pair(:,1)>pair(:,2),1)]; % ensures i<j -> leads to double pairs
pair=unique(pair,'rows'); % delete double pairs

% run recursion 7 to obtain matrix of required covariance elements
S=invertSparseCov(L, [pair(:,2) pair(:,1)], 1); 
% Slower by lees memory needed
% S = invertSparseCov(L, [pair(:,2) pair(:,1)], 0);



% composite marginal likelihood computation
llf=0;
Covm=zeros(2,2); % will contain covariance elements for each pair 

for g=1:length(pair);
    
    % select truncation thresholds for pair g
    vg=v([pair(g,1) pair(g,2)]);
    
    % store covariance elemnts for pair g in Covm
    Covm(1,1)=S(pair(g,1),pair(g,1));
    Covm(1,2)=S(pair(g,2),pair(g,1));
    Covm(2,1)=Covm(1,2);
    Covm(2,2)=S(pair(g,2),pair(g,2));
    
    % compute log-likelihood contribution
    llf=llf + log(mvncdf(vg,zeros(2,1),Covm));
end

llf=-llf;

end