function llf=LCML_uni(par,W,X,y)


% computes univariate composite marginal likelihood function (CML_1)


% par denotes Kx1 vector of paramters with par(1) being the spatial paramter rho!!
% W denotes NxN matrix of spatial weight => W should be declared as a sparse matrix!!
% X denotes NxK matrix of regressors
% y denotes Nx1 vector of a binary dependent variable

% the output argument llf denote the composite marginal likelihood function


warning('off', 'MATLAB:rankDeficientMatrix')
warning('off', 'MATLAB:nearlySingularMatrix')
warning('off', 'MATLAB:singularMatrix')


% determine number of observations
N=length(W);

% create Q matrix that allows to write the likelihood as an integral in terms of upper bounds (see Beron and Vijverbeg (2004) - Probit in a Spatial Context: A Monte Carlo Analysis)
Q = spdiags(1-2*y,0,N,N);

% compute vector of truncation thresholds (here we consider SAL model) and precision matrix 
Dinv=speye(N)-par(1)*W;
v=(-Q)*(Dinv\(X*par(2:end)));
H=Q'*(Dinv'*Dinv)*Q;


% compute covariance matrix elements
% SAMD permutation
ppp=symamd(H); % permutation vector
H=H(ppp,ppp);
v=v(ppp);

% Choleksy factorization
L = chol(H)';

% run recursion 7 to obtain matrix of required variances
S=invertSparseCov(L,1);
% Slower by lees memory needed
% S = invertSparseCov(L,0);


% composite marginal likelihood computation
llf= -sum(log(normcdf(v,zeros(N,1),sqrt(diag(S)))));

end